from django.urls import reverse
from django.shortcuts import render, redirect
from django.contrib import messages
from django.utils.translation import ugettext_lazy as _

from .conf import settings
from location_app.forms import LocationForm


def index(request):
	"""
	View for the 'LocationForm' form
	"""
	if request.method == "POST":
		form = LocationForm(request.POST)
		if form.is_valid():
			_ = form.save()
			if request.POST.get('next', None) != None:
				return redirect(request.POST['next'])

			return redirect(reverse('location'))
		else:
			messages.error(request, "Something went wrong! See the errors below or try to reload the page.")
	else:
		form = LocationForm()


	context = {
				'form': form,
				}

	context["map_options"] = settings.MAP_OPTIONS
	context["marker_options"] = settings.MARKER_OPTIONS
	# context["map_autocomplete_options"] = settings.MAP_AUTOCOMPLETE_OPTIONS
	return render(request, 'location_app/location.html', context)


from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from .models import *
from .conf import settings
from .widgets import LocationAppWidget


class LocationForm(forms.Form):
	"""
	ToDo: Make it such that location should be selected from dropdown otherwise show an error
	"""
	location = forms.CharField(required=False,label="Enter Location", help_text="Enter the location.")
	location.widget = LocationAppWidget()

	place = forms.CharField(label="Place")
	place_id = forms.CharField(label="Place Id")
	longitude = forms.FloatField(label="Longitude")
	latitude = forms.FloatField(label="Latitude")
	city = forms.CharField(required=False, label="City")
	state = forms.CharField(required=False, label="State")
	country = forms.CharField(label="Country")


	def save(self):
		plc_id = self.cleaned_data.get('place_id', None)
		if plc_id is not None:
			try:
				place_instance = Place.objects.get(place_id=plc_id)
			except:
				place_instance = None

			if place_instance is None:

				country_val = self.cleaned_data.get('country', None)
				if country_val is not None:
					try:
						country_instance = Country.objects.get(name=country_val)
					except Exception as e:
						country_instance = Country(name=country_val)
						country_instance.save()
				else:
					#we were expecting country value to be received, something went wrong
					raise ValidationError("country value was not received")


				state_val = self.cleaned_data.get('state', None)
				state_instance = None
				if state_val is not None:
					try:
						state_instance = AdministrativeArea.objects.get(name=state_val)
					except Exception as e:
						state_instance = AdministrativeArea(name=state_val, country=country_instance)
						state_instance.save()


				city_val = self.cleaned_data.get('city', None)
				city_instance = None
				if city_val is not None:
					try:
						city_instance = Locality.objects.get(name=city_val)
					except Exception as e:
						city_instance = Locality(name=city_val, state=state_instance, country=country_instance)
						city_instance.save()

				try:
					place_instance = Place(name=self.cleaned_data['place'],
						                   place_id=self.cleaned_data['place_id'],
		                                   longitude=self.cleaned_data['longitude'],
		                                   latitude=self.cleaned_data['latitude'],
		                                   city = city_instance,
		                                   state = state_instance,
		                                   country = country_instance,
		                                   )
					place_instance.save()
				except Exception as e:
					raise ValueError(repr(e))

			return place_instance

		else:
			raise ValidationError("place_id is a required field and was missing in the input")


from django.db import models

# Create your models here.
class Country(models.Model):
	name = models.CharField(unique=True,
							max_length=50,
							)
	class Meta:
		ordering = ['name']

	def __str__(self):
		return self.name


class AdministrativeArea(models.Model):
	name = models.CharField(null=True,
							blank=True,
							max_length=50,
							)
	country = models.ForeignKey(Country,
								on_delete=models.CASCADE,
								)

	class Meta:
		ordering = ['name']

	def display_country(self):
		return self.country

	def __str__(self):
		return self.name

class Locality(models.Model):
	name = models.CharField(max_length=50)
	state = models.ForeignKey(AdministrativeArea,
							  on_delete=models.CASCADE,
							  )
	country = models.ForeignKey(Country,
								on_delete=models.CASCADE,
								)

	class Meta:
		ordering = ['name', 'state', 'country']

	def display_state(self):
		return self.state

	def display_country(self):
		return self.country

	def __str__(self):
		return self.name

class Place(models.Model):
    name = models.CharField(max_length=100)
    place_id = models.CharField(max_length=256)
    longitude = models.FloatField()
    latitude = models.FloatField()
    country = models.ForeignKey(Country, on_delete=models.CASCADE)
    state = models.ForeignKey(AdministrativeArea, on_delete=models.CASCADE, null=True, blank=True)
    city = models.ForeignKey(Locality, on_delete=models.CASCADE, null=True, blank=True)

    class Meta:
        ordering = ['name']

    def display_country(self):
        return self.country

    def display_state(self):
        return self.state

    def display_city(self):
        return self.city

    def __str__(self):
        return self.name	


from django.urls import path
from django.contrib import admin
from django.shortcuts import render

from .models import *
from .forms import *

admin.site.register(Place)
admin.site.register(AdministrativeArea)
admin.site.register(Locality)
admin.site.register(Country)


from django.conf import settings as django_settings
from django.core.exceptions import ImproperlyConfigured


class AppSettings(object):
    prefix = 'LOC_APP'
    required_settings = ['MAPS_API_KEY']

    defaults = {
        'MAPS_API_KEY': None,
        # 'MAP_AUTOCOMPLETE_OPTIONS': '{"type": "establishment"}',
        # 'MAP_SIZE': "20rem",
        'MAP_OPTIONS': '{"center": {"lat": 40.7127753, "lng": -74.0059728}, "zoom": 16, "scale": 2, "draggable": false}',
        'MARKER_OPTIONS': '{"draggable": false}',
    }

    def __init__(self, django_settings):
        self.django_settings = django_settings

        for setting in self.required_settings:
            prefixed_name = '%s_%s' % (self.prefix, setting)
            if not hasattr(self.django_settings, prefixed_name):
                raise ImproperlyConfigured(
                    "The '%s' setting is required." % prefixed_name
                )

    def __getattr__(self, name):
        prefixed_name = '%s_%s' % (self.prefix, name)
        
        if hasattr(django_settings, prefixed_name):
            return getattr(django_settings, prefixed_name)

        if name in self.defaults:
            return self.defaults[name]
        
        raise AttributeError(
            "'AppSettings' object does not have a '%s' attribute" % name
        )


settings = AppSettings(django_settings)


from django import forms
from django.utils.translation import ugettext_lazy as _

from .conf import settings

class LocationAppWidget(forms.widgets.TextInput):
	def __init__(self, attrs=None):
		super(LocationAppWidget, self).__init__(attrs)

	class Media:
		js = ('//maps.googleapis.com/maps/api/js?key={}&libraries=places'.format(settings.MAPS_API_KEY),
        	  'location_app/js/location.js',
        	 )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages


readme = open('README.md').read()
history = open('CHANGELOG.md').read()

setup(
    author='Anmol Chachra',
    author_email='anmol.chachra@gmail.com',
    name='django-location-app',
    version='1.0.1',
    install_requires=['Django>=3.0'],
    python_requires='>=3.7',
    keywords='django geocomplete google maps places',
    classifiers=[
        'Environment :: Web Environment',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Framework :: Django',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],
    description='A Django app that helps you easily manage your own location data with autocomplete function. Connect your app to a completely normalized location database, avoid redundant Google Map API queries and save your hard earned money.',
    license='MIT',
    long_description=readme + '\n\n' + history,
    long_description_content_type='text/markdown',
    url='https://github.com/AnmolChachra/django-location-app',
    include_package_data=True,
    packages=find_packages(),
    package_data={
        'places': [
            'templates/location_app/*.html',
            'static/location_app/js/*.js',
        ],
    },
    project_urls={
        'Funding': 'https://donate.pypi.org',
        'Say Thanks!': 'https://saythanks.io/to/anmol.chachra@gmail.com',
        'Source': 'https://github.com/AnmolChachra/django-location-app/',
        'Tracker': 'https://github.com/AnmolChachra/django-location-app/issues',
    },
    zip_safe=False,
)